﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace NginxImport
{
    class Program
    {
        public static CultureInfo provider = CultureInfo.InvariantCulture;
        public static List<Thread> Threads = new List<Thread>();
        [STAThread]
        static void Main(string[] args)
        {
            var fod = new OpenFileDialog { Multiselect = true };
            if (fod.ShowDialog() != DialogResult.OK) return;
            foreach (var fileName in fod.FileNames)
            {
                //(new Thread(() => ParseFile(fileName))).Start();
                ParseFile(fileName);
            }
        }

        private static void ParseFile(string fileName)
        {
            Console.WriteLine(fileName);
            using (var fs = new StreamReader(fileName))
            {
                while (true)
                {
                    var data = new List<string>();
                    string line;
                    while (true)
                    {
                        line = fs.ReadLine();
                        if (line == null) break;
                        data.Add(line);
                        if (data.Count > 10000) break;
                    }
                    Console.WriteLine("offset " + fs.BaseStream.Position + " of " + fs.BaseStream.Length + " " + (fs.BaseStream.Position * 1.0 / fs.BaseStream.Length).ToString("P"));
                    if (data.Any())
                    {
                        // Parse10000(data);
                        (new Thread(() => Parse10000(data))).Start();

                        do
                        {
                            Thread.Sleep(100);
                        } while (Threads.Count + 1 >= Environment.ProcessorCount);
                    }
                    if (line == null) return;
                }
            }
        }


        private static void Parse10000(IEnumerable<string> data)
        {
            Threads.Add(Thread.CurrentThread);

            using (
                    var cn =
                        new SqlConnection(
                            "data source=.;initial catalog=nginxanal;integrated security=True;MultipleActiveResultSets=True;App=EntityFramework")
                    )
            using (var cmd = new SqlCommand(@"
if (select count(*) from [dbo].[nginxlog] where md5=@md5) = 0 begin

INSERT INTO [dbo].[nginxlog]
           ([md5]
           ,[IP]
           ,[Dt1]
           ,[URL]
           ,[RespCode]
           ,[Len]
           ,[Ref]
           ,[UA]
           ,[Meth]
           ,[Ver])
     VALUES
           (@md5
           ,@IP
           ,@Dt1
           ,@URL
           ,@RespCode
           ,@Len
           ,@Ref
           ,@UA
           ,@Meth
           ,@Ver)

end", cn))
            {
                cmd.Parameters.AddWithValue("md5", "");
                cmd.Parameters.AddWithValue("IP", "");
                cmd.Parameters.AddWithValue("Dt1", DateTime.Now);
                cmd.Parameters.AddWithValue("URL", "");
                cmd.Parameters.AddWithValue("RespCode", 0);
                cmd.Parameters.AddWithValue("Len", 0);
                cmd.Parameters.AddWithValue("Ref", "");
                cmd.Parameters.AddWithValue("UA", "");
                cmd.Parameters.AddWithValue("Meth", "");
                cmd.Parameters.AddWithValue("Ver", "");
                cn.Open();

                Task<int> task = null;
                foreach (var line in data)
                {
                    if (string.IsNullOrWhiteSpace(line)) continue;

                    try
                    {
                        var parts = line.ParseExact(@"{0} {7} {8} [{1}] ""{2}"" {3} {4} ""{5}"" ""{6}""");

                        string md5 = line.GetMd5();
                        string ip = parts[0];
                        DateTime dt1 = DateTime.ParseExact(parts[1], "dd/MMM/yyyy:HH:mm:ss zzz", provider);
                        string meth = parts[2].Split(new[] { ' ' })[0];
                        string ver = parts[2].Split(new[] { ' ' }).Last();
                        string url = parts[2].Remove(0, meth.Length);
                        url = url.Remove(url.Length - ver.Length);
                        url = url.Trim();
                        int code = int.Parse(parts[3]);
                        int len = int.Parse(parts[4]);
                        string Ref = parts[5];
                        string ua = parts[6];

                        cmd.Parameters["md5"].Value = md5;
                        cmd.Parameters["IP"].Value = ip;
                        cmd.Parameters["Dt1"].Value = dt1;
                        cmd.Parameters["URL"].Value = url;
                        cmd.Parameters["RespCode"].Value = code;
                        cmd.Parameters["Len"].Value = len;
                        cmd.Parameters["Ref"].Value = Ref;
                        cmd.Parameters["UA"].Value = ua;
                        cmd.Parameters["Meth"].Value = meth;
                        cmd.Parameters["Ver"].Value = ver;
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine("Err parce " + e + "\r\n\r\n" + line);
                        continue;
                    }

                    if (task != null && task.IsCompleted == false) task.Wait();
                    task = cmd.ExecuteNonQueryAsync();
                }
                cn.Close();
            }

            Memory.FlushMemory();

            Threads.Remove(Thread.CurrentThread);
        }

    }
}
